declare -A cpus
cpus[nhm-ex]=GenuineIntel-6-2E
cpus[nhm-ep]=GenuineIntel-6-1E
cpus[nhm-ep]=GenuineIntel-6-1A
cpus[wsm-ex]=GenuineIntel-6-2F
cpus[wsm-sp]=GenuineIntel-6-25
cpus[wsm-dp]=GenuineIntel-6-2C
cpus[snb]=GenuineIntel-6-2A
cpus[jkt]=GenuineIntel-6-2D
cpus[ivt]=GenuineIntel-6-3E
cpus[ivb]=GenuineIntel-6-3A
cpus[hsw]=GenuineIntel-6-45
cpus[slm]=GenuineIntel-6-37
cpus[bnl]=GenuineIntel-6-35
cpus[bdw]=GenuineIntel-6-3D
cpus[hsx]=GenuineIntel-6-3F
cpus[skl]=GenuineIntel-6-5E
cpus[bdw-de]=GenuineIntel-6-56
cpus[bdx]=GenuineIntel-6-4F
cpus[knl]=GenuineIntel-6-57
cpus[skx]=GenuineIntel-6-55-4
cpus[clx]=GenuineIntel-6-55-5
cpus[icl]=GenuineIntel-6-7E
cpus[icx]=GenuineIntel-6-6A
cpus[adl]=GenuineIntel-6-9A
cpus[spr]=GenuineIntel-6-8F
cpus[tgl]=GenuineIntel-6-8C
